from urllib import urlencode, quote
import base64
import hashlib

class Crypto(object):
    
    def __init__(self, config):
        self.config = config
        self.environment = self.config.environment
    
    def sign(self, method, url, parameters, secretKey):        
        base_string = self._build_base_string(method, url, parameters)
        return self._sign(base_string, secretKey)
    
    def _build_base_string(self, method, url, parameters):
        parameters = self.config.sort_dict(parameters)
        base_string = method + "&" + quote(url.lower(), '') + "&"
        
        qs = ''
        if parameters != {}:
            qs = urlencode(parameters)
        
        base_string = base_string + quote(qs)
        return base_string
    
    def _sign(self, base_string, secretKey):
        to_sha256 = hashlib.sha256()
        to_sha256.update(base_string + secretKey)
        string_sha256 = to_sha256.hexdigest()
        string_base64 = base64.b64encode(string_sha256)
        return string_base64


from configuration import Configuration
from resource import Resource

class Purchase(Resource):
    
    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)
    
    @staticmethod
    def find(id):
        return Configuration.gateway().purchase.find(id)
    
    @staticmethod
    def buy(id, attributes):
        return Configuration.gateway().purchase.buy(id, attributes)


class ErrorResult(object):
    
    def __init__(self, attributes):
        self.error = attributes['error']
        self.error_description = attributes['error_description']
    
    def __repr__(self):
        return "<%s: '%s'>" % (self.error, self.error_description)
    
    @property
    def is_success(self):
        
        return False


from configuration import Configuration
from resource import Resource

class Achievement(Resource):
    
    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)
    
    @staticmethod
    def find(player_id, id, attributes):
        return Configuration.gateway().achievement.find(player_id, id, attributes)
        


from gamerocket.result.error_result import ErrorResult
from gamerocket.result.successful_result import SuccessfulResult
from gamerocket.exceptions.not_found_error import NotFoundError
from achievement import Achievement

class AchievementGateway(object):
    
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config
    
    def find(self, player_id, id, attributes):
        if player_id == None or player_id.strip() == "" or id == None or id.strip() == "":
            raise NotFoundError()
        response = self.config.http().get("/players/" + player_id + "achievements/" + id, attributes)
        if "achievement" in response:
            return SuccessfulResult({"achievement": Achievement(self.gateway,response["achievement"])})
        elif "error" in response:
            return ErrorResult(response)
        else:
            pass


from attribute_getter import AttributeGetter

class Resource(AttributeGetter):
    def __init__(self, gateway, attributes):
        AttributeGetter.__init__(self, attributes)
        self.gateway = gateway
                


from action_gateway import ActionGateway
from achievement_gateway import AchievementGateway
from game_gateway import GameGateway
from player_gateway import PlayerGateway
from purchase_gateway import PurchaseGateway

class GamerocketGateway(object):
    def __init__(self, config):
        self.config = config
        self.achievement = AchievementGateway(self)
        self.action = ActionGateway(self)
        self.game = GameGateway(self)
        self.player = PlayerGateway(self)
        self.purchase = PurchaseGateway(self)


import os
import inspect

class Environment(object):
    
    def __init__(self, server, port, is_ssl, ssl_certificate):
        self.__server = server
        self.__port = port
        self.is_ssl = is_ssl
        self.ssl_certificate = ssl_certificate
    
    @property
    def base_url(self):
        return self.protocol + self.server_and_port
    
    @property
    def port(self):
        return int(self.__port)
    
    @property
    def protocol(self):
        if self.__port == "443":
            return "https://"
        else:
            return "http://"
    
    @property
    def server(self):
        return self.__server
    
    @property
    def server_and_port(self):
        return self.__server + ":" + self.__port
    
    @staticmethod
    def gamerocket_root():
        return os.path.dirname(inspect.getfile(Environment))

Environment.Development = Environment("localhost", os.getenv("GATEWAY_PORT") or "8280", False, None)
Environment.Production = Environment("api.gamerocket.io", "443", True, Environment.gamerocket_root() + "/ssl/www_gamerocket_io.ca.crt")



from gamerocket_gateway import GamerocketGateway
from configuration import Configuration
from environment import Environment
from version import Version
from attribute_getter import AttributeGetter

from achievement import Achievement
from achievement_gateway import AchievementGateway
from achievement_template import AchievementTemplate
from action import Action
from action_gateway import ActionGateway
from game import Game
from game_gateway import GameGateway
from player import Player
from player_gateway import PlayerGateway
from resource import Resource
from purchase import Purchase
from purchase_gateway import PurchaseGateway

from result.error_result import ErrorResult
from result.successful_result import SuccessfulResult



from gamerocket.result.error_result import ErrorResult
from gamerocket.result.successful_result import SuccessfulResult
from gamerocket.exceptions.not_found_error import NotFoundError
from game import Game

class GameGateway(object):
    
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config
    
    def find(self, id):
        if id == None or id.strip() == "":
            raise NotFoundError()
        response = self.config.http().get("/games/" + id)
        if "game" in response:
            return SuccessfulResult({"game": Game(self.gateway,response["game"])})
        elif "error" in response:
            return ErrorResult(response)
        else:
            pass


from gamerocket.exceptions.authentication_error import AuthenticationError
from gamerocket.exceptions.authorization_error import AuthorizationError
from gamerocket.exceptions.down_for_maintenance_error import DownForMaintenanceError
from gamerocket.exceptions.not_found_error import NotFoundError
from gamerocket.exceptions.server_error import ServerError
from gamerocket.exceptions.unexpected_error import UnexpectedError
from gamerocket.exceptions.upgrade_required_error import UpgradeRequiredError
from gamerocket import version

import gamerocket
from urllib import urlencode
import json

class Http(object):
    @staticmethod
    def is_error_status(status):
        return status not in [200, 201, 422]
    
    @staticmethod
    def raise_exception_from_status(status, message=None):
        if status == 401:
            raise AuthenticationError()
        elif status == 403:
            raise AuthorizationError(message)
        elif status == 404:
            raise NotFoundError()
        elif status == 426:
            raise UpgradeRequiredError()
        elif status == 500:
            raise ServerError()
        elif status == 503:
            raise DownForMaintenanceError()
        else:
            raise UnexpectedError("Unexpected HTTP_RESPONSE " + str(status))
        
    def __init__(self, config):
        self.config = config
        self.environment = self.config.environment
    
    def post(self, path, params={}):
        return self.__http_do("POST", path, params)
    
    def delete(self, path, params={}):
        return self.__http_do("DELETE", path, params)
    
    def get(self, path, params={}):
        return self.__http_do("GET", path, params)
    
    def put(self, path, params={}):
        return self.__http_do("PUT", path, params)
    
    def __http_do(self, http_verb, path, params=None):
        
        _params = params.copy()

        http_strategy = self.config.http_strategy()
        full_path = self.environment.base_url + self.config.api_version() + path
        
        _params['signature'] = self.config.crypto().sign(http_verb, full_path, _params, self.config.secretKey)
        
        request_body = ''
        if _params != {}:
            request_body = urlencode(_params)
        
        if http_verb == "GET":
            full_path += "?" + request_body
        elif http_verb == "DELETE":
            full_path += "?" + request_body
        
        status, response_body = http_strategy.http_do(http_verb, full_path, self.__headers(), request_body)
                
        if Http.is_error_status(status):
            Http.raise_exception_from_status(status)
        else:
            if len(response_body.strip()) == 0:
                return {}
            else:
                return json.loads(response_body)
    
    def __headers(self):
        return {
            "Accept" : "application/json",
            "Content-type" : "application/x-www-form-urlencoded",
            "User-Agent" : "Gamerocket Python " + version.Version,
            "X-ApiVersion" : gamerocket.configuration.Configuration.api_version()
        }



from authentication_error import AuthenticationError
from authorization_error import AuthorizationError
from down_for_maintenance_error import DownForMaintenanceError
from forged_query_string_error import ForgedQueryStringError
from invalid_signature_error import InvalidSignatureError
from not_found_error import NotFoundError
from server_error import ServerError
from unexpected_error import UnexpectedError
from upgrade_required_error import UpgradeRequiredError


from configuration import Configuration
from resource import Resource

class Player(Resource):
    
    def __repr__(self):
        detail_list = ["id", "name", "emailHash", "locale", "totalPointsAchievement", "ticksRemaining", "lastActivity", "dynProp"]
        return super(Player, self).__repr__(detail_list)
    
    @staticmethod
    def create(params={}):        
        return Configuration.gateway().player.create(params)
    
    @staticmethod
    def delete(player_id):        
        return Configuration.gateway().player.delete(player_id)
    
    @staticmethod
    def find(player_id):
        return Configuration.gateway().player.find(player_id)
    
    @staticmethod
    def update(player_id, params={}):
        return Configuration.gateway().player.update(player_id, params)
    
    @staticmethod
    def create_signature():
        return ["id", "name", "emailHash", "locale", "totalPointsAchievement", "ticksRemaining", "lastActivity", "dynProp"]
    
    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)


import sys
import os
from collections import OrderedDict
import gamerocket
import gamerocket.util.http_strategy.pycurl_strategy

class Configuration(object):
    
    @staticmethod
    def configure(environment, apiKey, secretKey):
        Configuration.environment = environment
        Configuration.apiKey = apiKey
        Configuration.secretKey = secretKey
        Configuration.use_unsafe_ssl = False
    
    @staticmethod
    def gateway():
        return gamerocket.gamerocket_gateway.GamerocketGateway(Configuration.instantiate())
    
    @staticmethod
    def instantiate():
        return Configuration(
            Configuration.environment,
            Configuration.apiKey,
            Configuration.secretKey
        )
    
    @staticmethod
    def api_version():
        return "1"
    
    def __init__(self, environment, apiKey, secretKey):
        self.environment = environment
        self.apiKey = apiKey
        self.secretKey = secretKey
        self._http_strategy = self.__determine_http_strategy()
    
    def crypto(self):
        return gamerocket.util.crypto.Crypto(self)
    
    def http(self):
        return gamerocket.util.http.Http(self)
    
    def http_strategy(self):
        if Configuration.use_unsafe_ssl:
            return gamerocket.util.http_strategy.httplib_strategy.HttplibStrategy(self, self.environment)
        else:
            return self._http_strategy
    
    def __determine_http_strategy(self):
        return gamerocket.util.http_strategy.pycurl_strategy.PycurlStrategy(self, self.environment)
    
    def __http_strategy_from_environment(self):
	return gamerocket.util.http_strategy.pycurl_strategy.PycurlStrategy(self, self.environment)
        
    def sort_dict(self, dict):
        dict_sorted = OrderedDict(sorted(dict.items(), key=lambda t: t[0]))
        return dict_sorted



from gamerocket.result.error_result import ErrorResult
from gamerocket.result.successful_result import SuccessfulResult
from gamerocket.exceptions.not_found_error import NotFoundError
from player import Player

class PlayerGateway(object):
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config
    
    def create(self, params={}):
        response = self.config.http().post("/games/" + self.config.apiKey + "/players", params)
        if "player" in response:
            return SuccessfulResult({"player": Player(self.gateway, response['player'])})
        elif "error" in response:
            return ErrorResult(response)
        else:
            pass
    
    def delete(self, player_id, params={}):
        response = self.config.http().delete("/players/" + player_id, params)
        if "status" in response:
            return SuccessfulResult(response)
        elif "error" in response:
            return ErrorResult(response)
        else:
            pass
    
    def find(self, player_id):
        if player_id == None or player_id.strip() == "":
            raise NotFoundError()
        response = self.config.http().get("/players/" + player_id)
        if "player" in response:
            return SuccessfulResult({"player": Player(self.gateway,response["player"])})
        elif "error" in response:
            return ErrorResult(response)
        else:
            pass
        
    def update(self, player_id, params={}):
        response = self.config.http().put("/players/" + player_id, params)
        if "player" in response:
            return SuccessfulResult({"player": Player(self.gateway,response["player"])})
        elif "error" in response:
            return ErrorResult(response)
        else:
            pass